if select(2, UnitClass("player")) ~= "WARLOCK" then return end
local GetSpellInfo = GetSpellInfo
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitStat = UnitStat
local UnitBuff = UnitBuff
local math_min = math.min

--Downranking exceptions: Death Coil, Siphon life, Searing Pain, Hellfire
--Untested assumed normal: UA, Conflagrate, Shadowfury
--No downranking: Lifetap, Drain soul, CoA

--TODO: Spellstone, Firestone

function DrDamage:PlayerData()

	--General
	local firestones = { [GetSpellInfo(54721)] = true, [GetSpellInfo(55152)] = true, [GetSpellInfo(55153)] = true, [GetSpellInfo(55154)] = true, [GetSpellInfo(55155)] = true, [GetSpellInfo(55156)] = true, [GetSpellInfo(55158)] = true }
	local spellstones = { [GetSpellInfo(55171)] = true, [GetSpellInfo(55175)] = true, [GetSpellInfo(55178)] = true, [GetSpellInfo(55188)] = true, [GetSpellInfo(55190)] = true, [GetSpellInfo(55194)] = true }
	
	self.Calculation["WARLOCK"] = function( calculation, ActiveAuras, Talents, spell, baseSpell )
		if Talents["Death's Embrace"] then
			if (UnitHealth("target") / UnitHealthMax("target")) <= 0.35 then
				calculation.dmgM = calculation.dmgM * ( 1 + 0.04 * Talents["Death's Embrace"] )
			end
		end
		local buff = self:GetWeaponBuff()
		if buff then
			if spellstones[buff] then
				calculation.dmgM_dot = calculation.dmgM_dot * 1.01 - 0.01 * (Talents["Shadow Mastery"] or 0) - 0.01 * (Talents["Emberstorm"] or 0)
			end
			if not baseSpell.eDot and firestones[buff] then
				calculation.dmgM_Add = calculation.dmgM * 1.01 - 0.01 * (Talents["Shadow Mastery"] or 0) - 0.01 * (Talents["Emberstorm"] or 0)
			end
		end
	end
	self.Calculation["Pandemic"] = function( calculation )
		calculation.canCrit = true
		calculation.critM = calculation.critM + 0.5
	end

	--Spell specific
	self.Calculation["Drain Life"] = function( calculation, ActiveAuras, BuffTalentRanks )
		if ActiveAuras["Soul Siphon"] and BuffTalentRanks["Soul Siphon"] then
			calculation.dmgM = calculation.dmgM * ( 1 + math_min( BuffTalentRanks["Soul Siphon"] * 0.09, ActiveAuras["Soul Siphon"] * BuffTalentRanks["Soul Siphon"] * 0.03 ) )	
		end
		if BuffTalentRanks["Death's Embrace"] then
			if (UnitHealth("player") / UnitHealthMax("player")) <= 0.2 then
				calculation.dmgM = calculation.dmgM * ( 1 + 0.1 * BuffTalentRanks["Death's Embrace"] )
			end
		end
	end
	self.Calculation["Drain Soul"] = function ( calculation, ActiveAuras, BuffTalentRanks, spell )
		if ActiveAuras["Soul Siphon"] and BuffTalentRanks["Soul Siphon"] then
			calculation.dmgM = calculation.dmgM * ( 1 + math_min( BuffTalentRanks["Soul Siphon"] * 0.09, ActiveAuras["Soul Siphon"] * BuffTalentRanks["Soul Siphon"] * 0.03 ) )	
		end	
		if UnitHealth("target") ~= 0 then
			if (UnitHealth("target") / UnitHealthMax("target")) <= 0.25 then
				calculation.minDam = calculation.minDam * 4
				calculation.maxDam = calculation.maxDam * 4
			end
		end
	end
	--TODO: Fix min and max bonus to be based on player level
	self.Calculation["Incinerate"] = function( calculation, ActiveAuras, _, spell )
		--Glyph of Incinerate
		if self:HasGlyph(56242) then
			calculation.dmgM_Add = calculation.dmgM_Add + 0.05
		end
		if ActiveAuras["Immolate"] then
			calculation.minDam = calculation.minDam + spell.iMin + spell.minBonus
			calculation.maxDam = calculation.maxDam + spell.iMax + spell.maxBonus
		end
	end
	self.Calculation["Life Tap"] = function ( calculation, ActiveAuras, BuffTalentRanks, spell )
		calculation.minDam = calculation.minDam + spell.spiritFactor * select(2,UnitStat("player",5))
		calculation.maxDam = calculation.maxDam + spell.spiritFactor * select(2,UnitStat("player",5))
	end
	self.Calculation["Immolate"] = function( calculation )
		--Glyph of Immolate
		if self:HasGlyph(56228) then
			calculation.dmgM_dot_Add = calculation.dmgM_dot_Add + 0.1
		end
	end
	self.Calculation["Curse of Agony"] = function( calculation )
		--Glyph of Curse of Agony
		--TODO: Fix for more damage
		if self:HasGlyph(56241) then
			calculation.eDuration = calculation.eDuration + 4
		end
	end
	self.Calculation["Searing Pain"] = function( calculation )
		--Glyph of Searing Pain
		if self:HasGlyph(56226) then
			calculation.critM = calculation.critM + 0.1
		end
	end
	self.Calculation["Corruption"] = function( calculation )
		--Glyph of Siphon Life
		if self:HasGlyph(56216) then
			calculation.leechBonus = calculation.leechBonus * 1.25
		end
	end
	--TODO: Fix aura rank detection
	local Immolate = GetSpellInfo(348)
	local Shadowflame = GetSpellInfo(47897)
	self.Calculation["Conflagrate"] = function( calculation, ActiveAuras, Talents, spell )
		if ActiveAuras["Immolate"] then
			calculation.dmgM_custom = true
			--Hotfix nerf 0.7 * Buffs (Master Demonology * MC * CoE..)  * ( Emberstorm + Firestone + T8 set bonus ) * ( Improved Immolate + Afermath + Glyph )
			--PTR F&B removed (15% multiplicative to incinerate), coefficient 12/15
			calculation.dmgM = 0.7 * calculation.dmgM * (1 + calculation.dmgM_Add + ((self:GetSetAmount( "Tier 8" ) >= 2) and 0.1 or 0)) * (1 + (Talents["Improved Immolate"] or 0) + calculation.dmgM_dot_Add + (self:HasGlyph(56228) and 0.1 or 0))
			calculation.spellDmgM = 1 + (Talents["Fire and Brimstone"] or 0)
			calculation.minDam = self.spellInfo[Immolate][11].hybridDotDmg
			calculation.maxDam = calculation.minDam
		elseif ActiveAuras["Shadowflame"] then
			calculation.spellDmgM = 8/15
			calculation.minDam = self.spellInfo[Shadowflame]["Secondary"][2].hybridDotDmg
			calculation.maxDam = calculation.minDam
		end
	end
	
	--Set bonuses
	--Sets
	self.SetBonuses["Tier 8"] = { 46135, 46136, 46137, 46139, 46140, 45417, 45419, 45420, 45421, 45422 }
	
	--Effects
	self.SetBonuses["Immolate"] = function( calculation )
		if self:GetSetAmount( "Tier 8" ) >= 2 then
			calculation.dmgM_Add = calculation.dmgM_Add + 0.10
		end
	end
	self.SetBonuses["Unstable Affliction"] = function( calculation )
		if self:GetSetAmount( "Tier 8" ) >= 2 then
			calculation.dmgM_Add = calculation.dmgM_Add + 0.20
		end
	end
	self.SetBonuses["Shadow Bolt"] = function( calculation )
		if self:GetSetAmount( "Tier 8" ) >= 4 then
			calculation.critPerc = calculation.critPerc + 5
		end		
	end
	self.SetBonuses["Incinerate"] = function( calculation )
		if self:GetSetAmount( "Tier 8" ) >= 4 then
			calculation.critPerc = calculation.critPerc + 5
		end		
	end

	--AURA
	--Pyroclasm
	--Demon Armor
	--Molten Core
	--Master Demonologist
	--Demonic Soul
	self.PlayerAura[GetSpellInfo(63245)] = { School = { "Fire", "Shadow" }, Ranks = 3, Value = 0.02 }
	self.PlayerAura[GetSpellInfo(706)] = { ModType = "Special" }
	self.PlayerAura[GetSpellInfo(47383)] = { School = "Fire", Value = 0.1 }
	self.PlayerAura[GetSpellInfo(23785)] = { School = { "Fire", "Shadow" }, ModType = "Special" }
	self.PlayerAura[GetSpellInfo(61595)] = { Spell = { GetSpellInfo(686),  GetSpellInfo(29722) }, ModType = "critPerc", Value = 10 }
	--Haunt
	--Shadow Embrace
	--Immolate
	--Shadowflame
	self.TargetAura[GetSpellInfo(48181)] = { School = "Shadow", ModType = "Special" }
	self.TargetAura[GetSpellInfo(32386)] = { School = "Shadow", ModType = "Special" }
	self.TargetAura[GetSpellInfo(348)] = { Spell = { GetSpellInfo(29722), GetSpellInfo(17962) }, ModType = "ActiveAura", ActiveAura = "Immolate", }
	self.TargetAura[GetSpellInfo(61290)] = { Spell = GetSpellInfo(17962), ModType = "ActiveAura", ActiveAura = "Shadowflame", }
	--Corruption
	--Death Coil
	--Curse of Agony
	--Curse of Doom
	--Curse of Weakness
	--Curse of Tongues
	--Curse of Exhaustion
	--Fear
	--Drain Soul
	--Drain Mana
	--Unstable Affliction
	--Seed of Corruption
	--Howl of Terror
	self.TargetAura[GetSpellInfo(172)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(6789)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(980)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(603)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(702)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(1714)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(18223)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(5782)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(1120)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(5138)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(30108)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(27243)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	self.TargetAura[GetSpellInfo(5484)] = 	{ ActiveAura = "Soul Siphon", Spell = GetSpellInfo(689), ModType = "ActiveAura" }
	
	--Demon Armor
	self.Calculation[GetSpellInfo(706)] = function( calculation, _, BuffTalentRanks )
		calculation.leechBonus = calculation.leechBonus  * (1.2 + 0.02 * (BuffTalentRanks["Demonic Aegis"] or 0))
	end
	
	--Shadow Embrace
	self.Calculation[GetSpellInfo(32386)] = function( calculation, ActiveAuras, BuffTalentRanks, _, apps )
		if BuffTalentRanks["Shadow Embrace"] and apps then
			calculation.dmgM_dot = calculation.dmgM_dot * (1 + BuffTalentRanks["Shadow Embrace"] * apps)
		end
		if ActiveAuras["Soul Siphon"] then
			ActiveAuras["Soul Siphon"] = ActiveAuras["Soul Siphon"] + 1
		end
	end
	
	--Haunt
	self.Calculation[GetSpellInfo(48181)] = function( calculation )
		--Glyph of Haunt
		calculation.dmgM_dot = calculation.dmgM_dot * (1.2 + (self:HasGlyph(63302) and 0.03 or 0))
	end
	
	--[[ Fix manual applying later..
	local felarmor = GetSpellInfo(28176)
	self.Calculation[GetSpellInfo(28176)] = function( calculation, _, BuffTalentRanks )
		if not UnitBuff("player", felarmor) then
			calculation.spellDmg = calculation.spellDmg + 100 * (1 + 0.1 * (BuffTalentRanks["Demonic Aegis"] or 0)) + 0.3 * select(2, UnitStat("player", 5))
		end
	end
	--]]
	
	--Master demonologist
	local imp = GetSpellInfo(3110)
	local succubus = GetSpellInfo(7814)
	self.Calculation[GetSpellInfo(23785)] = function ( calculation, _, Talents )
		if calculation.spellSchool == "Fire" and GetSpellInfo(imp) or calculation.spellSchool == "Shadow" and GetSpellInfo(succubus) then
			calculation.dmgM = calculation.dmgM * (1 + (Talents["Master Demonologist"] or 0))
		end
	end
		

	self.spellInfo = {
		[GetSpellInfo(172)] = {	--Base OK, Increase OK, Coefficient X
					["Name"] = "Corruption",
					[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 1.2, castTime = 2, eDot = true, eDuration = 18, sTicks = 3, },
					[1] = { 40, 40, spellLevel = 4, eDuration = 12, Downrank = -1, },
					[2] = { 90, 90, spellLevel = 14, eDuration = 15, },
					[3] = { 222, 222, spellLevel = 24, },
					[4] = { 324, 324, spellLevel = 34, },
					[5] = { 486, 486, spellLevel = 44, }, 
					[6] = { 666, 666, spellLevel = 54, },
					[7] = { 822, 822, spellLevel = 60, },
					[8] = { 900, 900, spellLevel = 65, },
					[9] = { 984, 984, spellLevel = 71, },
					[10] = { 1080, 1080, spellLevel = 77, },
		},
		[GetSpellInfo(686)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Shadow Bolt",
					[0] = { School = { "Shadow", "Destruction" }, castTime = 3, canCrit = true, BaseIncrease = true, LevelIncrease = 5 },
					[1] = { 12, 16, 31, 32, spellLevel = 1, castTime = 1.7, Downrank = -1, },
					[2] = { 23, 29, 44, 45, spellLevel = 6, castTime = 2.2, },
					[3] = { 48, 56, 61, 62, spellLevel = 12, castTime = 2.8, },
					[4] = { 86, 98, 72, 72, spellLevel = 20, },
					[5] = { 142, 162, 83, 84, spellLevel = 28, },
					[6] = { 204, 230, 83, 84, spellLevel = 36, },
					[7] = { 281, 315, 82, 83, spellLevel = 44, },
					[8] = { 360, 402, 72, 73, spellLevel = 52, }, 
					[9] = { 455, 507, 60, 60, spellLevel = 60, },
					[10] ={ 482, 538, 62, 62, spellLevel = 60, },
					[11] ={ 541, 603, 37, 38, spellLevel = 69, },
					[12] ={ 596, 664, 22, 23, spellLevel = 74, },
					[13] ={ 690, 770, 4, 5, spellLevel = 79, },
		},
		[GetSpellInfo(1454)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Life Tap",
					[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 0, BaseIncrease = true, LevelIncrease = 10, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, NoDownRank = true, },
					[1] = { 0, 0, 10, 10, spellLevel = 6, spiritFactor = 1.0, },
					[2] = { 6, 6, 10, 10, spellLevel = 16, spiritFactor = 1.5, },
					[3] = { 24, 24, 10, 10, spellLevel = 26, spiritFactor = 2.0, },
					[4] = { 37, 37, 10, 10, spellLevel = 36, spiritFactor = 2.5, },
					[5] = { 42, 42, 10, 10, spellLevel = 46, spiritFactor = 3.0, },
					[6] = { 500, 500, 10, 10, spellLevel = 56, spiritFactor = 3.0, },
					[7] = { 710, 710, 10, 10, spellLevel = 68, spiritFactor = 3.0, },
					[8] = { 1490, 1490, 0, 0, spellLevel = 80, spiritFactor = 3.0, },
		},
		[GetSpellInfo(6229)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Shadow Ward",
					[0] = { School = "Shadow", bonusFactor = 0.3, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, NoDownRank = true },
					[1] = { 290, 290, spellLevel = 32, },
					[2] = { 470, 470, spellLevel = 42, },
					[3] = { 675, 675, spellLevel = 52, },
					[4] = { 875, 875, spellLevel = 60, },
					[5] = { 2750, 2750, spellLevel = 72, },
					[6] = { 3300, 3300, spellLevel = 78, },
		},		
		[GetSpellInfo(18220)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Dark Pact",
					[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 0.96, NoSchoolTalents= true, NoDPS = true, NoTooltip = true, NoAura = true, Unresistable = true, NoDownRank = true, },
					[1] = { 305, 305, spellLevel = 40, },
					[2] = { 440, 440, spellLevel = 50, },
					[3] = { 545, 545, spellLevel = 60, },
					[4] = { 700, 700, spellLevel = 70, },
					[5] = { 1200, 1200, spellLevel = 80, },
		},			
		[GetSpellInfo(980)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Curse of Agony",
					[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 1.20, eDot = true, eDuration = 24, sTicks = 2, NoDownRank = true, },
					[1] = { 84, 84, spellLevel = 8,  },
					[2] = { 180, 180, spellLevel = 18, },
					[3] = { 324, 324, spellLevel = 28, }, 
					[4] = { 504, 504, spellLevel = 38, }, 
					[5] = { 780, 780, spellLevel = 48, }, 
					[6] = { 1044, 1044, spellLevel = 58, },
					[7] = { 1356, 1356, spellLevel = 67, }, 
					[8] = { 1440, 1440, spellLevel = 73, },
					[9] = { 1740, 1740, spellLevel = 79, }, 
		},
		[GetSpellInfo(603)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Curse of Doom",
					[0] = { School = { "Shadow", "Affliction", }, bonusFactor = 2, NoSchoolTalents = true, eDot = true, eDuration = 60, },
					[1] = { 3200, 3200, spellLevel = 60, },
					[2] = { 4200, 4200, spellLevel = 70, },
					[3] = { 7300, 7300, spellLevel = 80, },
		},
		[GetSpellInfo(6789)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Death Coil",
					[0] = { School = { "Shadow", "Affliction", }, sFactor = 0.5, Cooldown = 120, Leech = true, NoDoom = true, BaseIncrease = true, LevelIncrease = 7, Downrank = 1 },
					[1] = { 244, 244, 83, 84, spellLevel = 42, },
					[2] = { 319, 319, 78, 78, spellLevel = 50, }, 
					[3] = { 400, 400, 66, 66, spellLevel = 58, },
					[4] = { 519, 519, 40, 41, spellLevel = 68, },
					[5] = { 670, 670, 28, 28, spellLevel = 73, },
					[6] = { 790, 790, 10, 10, spellLevel = 78, },
					--Note: Downrank: +1 -> exception
		},
		[GetSpellInfo(689)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Drain Life",
					[0] = { School = { "Shadow", "Affliction", }, castTime = 5, Leech = true, sHits = 5, sFactor = 0.5, },
					[1] = { 10, 10, spellLevel = 14, },
					[2] = { 17, 17, spellLevel = 22, },
					[3] = { 29, 29, spellLevel = 30, },
					[4] = { 41, 41, spellLevel = 38, },
					[5] = { 55, 55, spellLevel = 46, },
					[6] = { 71, 71, spellLevel = 54, },
					[7] = { 87, 87, spellLevel = 62, },
					[8] = { 108, 108, spellLevel = 69, },
					[9] = { 133, 133, spellLevel = 78, },
		},
		[GetSpellInfo(1120)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Drain Soul",
					[0] = { School = { "Shadow", "Affliction", }, castTime = 15, sFactor = 0.5, sTicks = 3, NoDownRank = true, },
					[1] = { 55, 55, spellLevel = 10, },
					[2] = { 155, 155, spellLevel = 24, },
					[3] = { 295, 295, spellLevel = 38, },
					[4] = { 455, 455, spellLevel = 52, },
					[5] = { 620, 620, spellLevel = 67, },
					[6] = { 710, 710, spellLevel = 77, },
		},
		[GetSpellInfo(5676)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Searing Pain",
					[0] = { School = { "Fire", "Destruction" }, canCrit = true, BaseIncrease = true, LevelIncrease = 7, Downrank = 1 },
					[1] = { 34, 42, 43, 44, spellLevel = 18, },
					[2] = { 59, 71, 54, 54, spellLevel = 26, },
					[3] = { 86, 104, 55, 56, spellLevel = 34, },
					[4] = { 122, 146, 57, 57, spellLevel = 42, },
					[5] = { 158, 188, 51, 51, spellLevel = 50, },
					[6] = { 204, 240, 44, 44, spellLevel = 58, },
					[7] = { 243, 287, 36, 36, spellLevel = 65, },
					[8] = { 270, 320, 28, 28, spellLevel = 70, },
					[9] = { 295, 349, 24, 25, spellLevel = 74, },
					[10] = { 343, 405, 4, 5, spellLevel = 79, },
					--Note: Downranking +1 -> exception
		},
		[GetSpellInfo(6353)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Soul Fire",
					[0] = { School = { "Fire", "Destruction" }, bonusFactor = 1.15, castTime=6, canCrit = true, CastMod = 0.4, BaseIncrease = true, Cooldown = 60,  LevelIncrease = 7 },
					[1] = { 623, 783, 92, 93, spellLevel = 48, },
					[2] = { 703, 881, 74, 75, spellLevel = 56, },
					[3] = { 839, 1051, 56, 56, spellLevel = 64, },
					[4] = { 1003, 1257, 38, 38, spellLevel = 70, },
					[5] = { 1137, 1423, 20, 20, spellLevel = 75, },
					[6] = { 1323, 1657, 0, 0, spellLevel = 80, },
		},
		[GetSpellInfo(17877)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Shadowburn",
					[0] = { School = { "Shadow", "Destruction" }, Cooldown = 15, canCrit = true, BaseIncrease = true,  LevelIncrease = 7 },
					[1] = { 87, 99, 72, 72, spellLevel = 20, },
					[2] = { 115, 131, 78, 79, spellLevel = 24, },
					[3] = { 186, 210, 86, 87, spellLevel = 32, },
					[4] = { 261, 293, 88, 88, spellLevel = 40, },
					[5] = { 350, 392, 83, 84, spellLevel = 48, },
					[6] = { 450, 502, 72, 72, spellLevel = 56, },
					[7] = { 518, 578, 57, 58, spellLevel = 63, },
					[8] = { 597, 665, 38, 38, spellLevel = 70, },
					[9] = { 662, 738, 25, 25, spellLevel = 75, },
					[10] = { 775, 865, 0, 0, spellLevel = 80, },
		},
		[GetSpellInfo(348)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Immolate",
					[0] = { School = { "Fire", "Destruction" }, canCrit = true, castTime = 2, bonusFactor = 0.2, eDuration = 15, sTicks = 3, BaseIncrease = true,  LevelIncrease = 5 },
					[1] = { 8, 8, 47, 48, hybridDotDmg = 20, spellLevel = 1, Downrank = -1 },
					[2] = {	19, 19, 63, 63, hybridDotDmg = 40, spellLevel = 10, }, 	 
					[3] = { 45, 45, 90, 90, hybridDotDmg = 90, spellLevel = 20, },	 
					[4] = { 90, 90, 105, 105, hybridDotDmg = 165, spellLevel = 30, }, 
					[5] = { 134, 134, 90, 90, hybridDotDmg = 255, spellLevel = 40, },
					[6] = { 192, 192, 96, 96, hybridDotDmg = 365, spellLevel = 50, }, 
					[7] = { 258, 258, 76, 76, hybridDotDmg = 485, spellLevel = 60, },
					[8] = { 279, 279, 78, 78, hybridDotDmg = 510, spellLevel = 60, },
					[9] = { 327, 327, 47, 48, hybridDotDmg = 615, spellLevel = 69, }, --Guess --DALLYTEMP
					[10] = { 370, 370, 25, 25, hybridDotDmg = 695, spellLevel = 75, },
					[11] = { 460, 460, 0, 0, hybridDotDmg = 785, spellLevel = 80, },
		},
		[GetSpellInfo(1949)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Hellfire",
					[0] = { School = "Fire", castTime = 15, sHits = 15, sFactor = 1/2, BaseIncrease = true, LevelIncrease = 11, Downrank = 5, },
					[1] = { 83, 83, 4, 4, spellLevel = 30, },
					[2] = { 139, 139, 5, 5, spellLevel = 42, },
					[3] = { 208, 208, 7, 7, spellLevel = 54, },
					[4] = { 306, 306, 8, 8, spellLevel = 68, },
					[5] = { 451, 451, 2, 2, spellLevel = 78, },
					--Note: Downrank +5 -> exception
		},
		[GetSpellInfo(5740)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Rain of Fire",
					[0] = { School = "Fire", castTime = 8, bonusFactor = 2.7728, sTicks = 2, BaseIncrease = true, LevelIncrease = 11, },
					[1] = { 240, 240, 72, 72, spellLevel = 20, },
					[2] = { 544, 544, 72, 72, spellLevel = 34, },
					[3] = { 880, 880, 68, 68, spellLevel = 46, },
					[4] = { 1284, 1284, 52, 52, spellLevel = 58, },
					[5] = { 1808, 1808, 32, 32, spellLevel = 69, }, --Guess --DALLYTEMP
					[6] = { 2152, 2152, 48, 48, spellLevel = 72, },
					[7] = { 2700, 2700, 8, 8, spellLevel = 79, },
		},
		[GetSpellInfo(30108)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Unstable Affliction",
					[0] = { School = { "Shadow", "Affliction", }, eDot = true, eDuration = 18, sTicks = 3, },
					[1] = { 550, 550, spellLevel = 50, },
					[2] = { 700, 700, spellLevel = 60, },
					[3] = { 875, 875, spellLevel = 70, },
					[4] = { 985, 985, spellLevel = 75, },
					[5] = { 1150, 1150, spellLevel = 80, },
		},
		[GetSpellInfo(17962)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Conflagrate",
					[0] = { School = { "Fire", "Destruction" }, canCrit = true, Cooldown = 10 },
					[1] = { 0, 0, spellLevel = 40, },
		},
		[GetSpellInfo(27243)] = { --Base OK, Increase OK (rank 2 upper?), Coefficient X
					["Name"] = "Seed of Corruption",
					[0] = { School = { "Shadow", "Affliction", }, castTime = 2, canCrit = true, bonusFactor = 0.2129, dotFactor = 1.5, eDuration = 18, sTicks = 3, },
					[1] = { 1110, 1290, hybridDotDmg = 1044, spellLevel = 70, },
					[2] = { 1383, 1612, hybridDotDmg = 1296, spellLevel = 75, },
					[3] = { 1633, 1897, hybridDotDmg = 1518, spellLevel = 80, },
		},
		[GetSpellInfo(30283)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Shadowfury",
					[0] = { School = { "Shadow", "Destruction" }, canCrit = true, bonusFactor = 0.1932, Cooldown = 20, BaseIncrease = true,  LevelIncrease = 9, },
					[1] = { 343, 407, 48, 48, spellLevel = 50, },
					[2] = { 459, 547, 38, 38, spellLevel = 60, },
					[3] = { 612, 728, 22, 22, spellLevel = 70, },
					[4] = { 822, 978, 15, 15, spellLevel = 75, },
					[5] = { 968, 1152, 0, 0, spellLevel = 80, },
		},
		[GetSpellInfo(32231)] = { --Base OK, Increase OK (TODO: Immolate bonus), Coefficient X
					["Name"] = "Incinerate",
					[0] = { School = { "Fire", "Destruction" }, canCrit = true, castTime = 2.5, BaseIncrease = true, LevelIncrease = 5, },
					[1] = { 403, 467, 41, 42, iMin = 100, iMax = 116, minBonus = 11, maxBonus = 12, spellLevel = 64, },
					[2] = { 444, 514, 28, 28, iMin = 111, iMax = 128, minBonus = 7, maxBonus = 8, spellLevel = 70, },
					[3] = { 485, 563, 18, 18, iMin = 121, iMax = 140, minBonus = 4, maxBonus = 6, spellLevel = 74, },
					[4] = { 582, 676, 0, 0, iMin = 145, iMax = 169, minBonus = 0, maxBonus = 0, spellLevel = 80, },
		},
		[GetSpellInfo(47897)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Shadowflame",
					[0] = { School = { "Shadow", "Destruction" }, canCrit = true, Cooldown = 15, BaseIncrease = true, LevelIncrease = 5 },
					[1] = { 520, 568, 10, 10, spellLevel = 75, },
					[2] = {	615, 671, 0, 0, spellLevel = 80, },
			["Secondary"] = {
					["Name"] = "Shadowflame",
					[0] = { School = { "Fire", "Destruction" }, castTime=0, eDuration = 8, sTicks = 2, Cooldown = 15 },
					[1] = { 0, 0, hybridDotDmg = 544, spellLevel = 75,  },
					[2] = {	0, 0, hybridDotDmg = 644, spellLevel = 80, },
			},
		},
		[GetSpellInfo(48181)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Haunt",
					[0] = { School = { "Shadow", "Affliction" }, canCrit = true, castTime = 1.5, Cooldown = 8 },
					[1] = { 405, 473, spellLevel = 60, },
					[2] = { 487, 569, spellLevel = 70, },
					[3] = { 550, 642, spellLevel = 75, },
					[4] = { 645, 753, spellLevel = 80, },
		},
		[GetSpellInfo(50796)] = { --Base OK, Increase OK, Coefficient X
					["Name"] = "Chaos Bolt",
					[0] = { School = { "Fire", "Destruction" }, castTime = 2.5, canCrit = true, Unresistable = true, Cooldown = 12, BaseIncrease = true, LevelIncrease = 5 },
					[1] = { 837, 1061, 110, 110, spellLevel = 60, },
					[2] = { 1077, 1367, 55, 55, spellLevel = 70, },
					[3] = { 1217, 1545, 27, 28, spellLevel = 75, },
					[4] = { 1429, 1813, 0, 0, spellLevel = 80, },
		},
	}
	self.talentInfo = {
		--AFFLICTION
		--Improved Curse of Agony
		[GetSpellInfo(18827)] = { 	[1] = { Effect = 0.05, Spells = "Curse of Agony" }, },
		--Suppression
		[GetSpellInfo(18174)] = { 	[1] = { Effect = 1, Spells = "All", ModType = "hitPerc", }, },
		--Improved Corruption
		[GetSpellInfo(17810)] = {	[1] = { Effect = 0.02, Spells = "Corruption" },
						[2] = { Effect = 1, Spells = "Seed of Corruption", ModType = "critPerc", }, },
		--Improved Life Tap
		[GetSpellInfo(18182)] = { 	[1] = { Effect = 0.1, Spells = "Life Tap", }, },
		--Soul Siphon
		[GetSpellInfo(17804)] = { 	[1] = { Effect = 1, Spells = { "Drain Soul", "Drain Life" }, ModType = "Amount", Value = "Soul Siphon" }, },
		--Empowered Corruption
		[GetSpellInfo(32381)] = { 	[1] = { Effect = 0.12, Spells = "Corruption", ModType = "SpellDamage" }, },
		--Shadow Embrace
		[GetSpellInfo(32385)] = {	[1] = { Effect = 0.01, Spells = "All", ModType = "Amount", Value = "Shadow Embrace" }, },
		--Siphon Life
		[GetSpellInfo(63108)] = {	[1] = { Effect = 0.01, Spells = { "Corruption", "Unstable Affliction" }, }, 
						[2] = { Effect = 0.01, Spells = "Seed of Corruption", ModType = "dmgM_dot_Add" }, 
						[3] = { Effect = 0.4, Spells = "Corruption", ModType = "leechBonus" }, },
		--Shadow Mastery
		[GetSpellInfo(18271)] = { 	[1] = { Effect = 0.03, Spells = "Shadow" },
						[2] = { Effect = 1, Spells = "Shadow", ModType = "Amount", Value = "Shadow Mastery" }, },
		--Contagion
		[GetSpellInfo(30060)] = { 	[1] = { Effect = 0.01, Spells = { "Curse of Agony", "Corruption", "Seed of Corruption" }, }, },
		--Malediction
		[GetSpellInfo(32477)] = {	[1] = { Effect = 0.01, Spells = "All", Multiply = true },
						[2] = { Effect = 3, Spells = { "Unstable Affliction", "Corruption" }, ModType = "critPerc" }, },
		--Death's Embrace				
		[GetSpellInfo(47198)] = {	[1] = { Effect = 1, Spells = "Shadow", ModType = "Amount", Value = "Death's Embrace", }, },
		--Pandemic
		[GetSpellInfo(58435)] = {	[1] = { Effect = 1, Spells = { "Corruption", "Unstable Affliction" }, ModType = "Pandemic" }, },
		--Everlasting Affliction
		[GetSpellInfo(47201)] = {	[1] = { Effect = 0.06, Spells = "Corruption", ModType = "SpellDamage" }, 
						[2] = { Effect = 0.05, Spells = "Unstable Affliction", ModType = "SpellDamage" }, },
		--DEMONOLOGY
		--Demonic Aegis
		[GetSpellInfo(30143)] = { 	[1] = { Effect = 1, Spells = "All", ModType = "Amount", Value = "Demonic Aegis" }, },
		--Master Demonologist
		[GetSpellInfo(23785)] = {	[1] = { Effect = 0.01, Spells = "All", ModType = "Amount", Value = "Master Demonologist" }, },
		--TODO: Determine if this is needed: Master Conjuror
		--[GetSpellInfo(18767)] = {	[1] = { Effect = 0.015, Spells = "All", ModType = "Amount", Value = "Master Conjuror" }, },
		
		--DESTRUCTION:
		--Improved Shadow Bolt
		[GetSpellInfo(17793)] = {	[1] = { Effect = 0.01, Spells = "Shadow Bolt" }, },
		--Aftermath
		[GetSpellInfo(18119)] = {	[1] = { Effect = 0.03, Spells = { "Conflagrate", "Immolate", }, ModType = "dmgM_dot_Add", }, },
		--Ruin
		[GetSpellInfo(17959)] = { 	[1] = { Effect = 0.1, Spells = "Destruction", ModType = "critM", }, },
		--Improved Searing Pain
		[GetSpellInfo(17927)] = { 	[1] = { Effect = { 4, 7, 10, }, Spells = "Searing Pain", ModType = "critPerc", }, },
		--Improved Immolate
		[GetSpellInfo(17815)] = { 	[1] = { Effect = 0.1, Spells = "Immolate" }, 
						[2] = { Effect = 0.1, Spells = "Conflagrate", ModType = "Amount", Value = "Improved Immolate" }, },
		--Devastation
		[GetSpellInfo(18130)] = { 	[1] = { Effect = 5, Spells = "Destruction", ModType = "critPerc", }, },
		--Emberstorm	
		[GetSpellInfo(17954)] = { 	[1] = { Effect = 0.03, Spells = "Fire", Mod = { "Incinerate", 3, 16, "castTime", function(v,c,t) return (v+c*0.05-t*0.05) end }, },
						[2] = { Effect = 1, Spells = "Fire", ModType = "Amount", Value = "Emberstorm" }, },		
		--Shadow and Flame	
		[GetSpellInfo(30288)] = { 	[1] = { Effect = 0.04, Spells = { "Shadow Bolt", "Chaos Bolt", "Incinerate" }, ModType = "SpellDamage", Multiply = true }, },
		--Fire and Brimstone
		[GetSpellInfo(47266)] = {	[1] = { Effect = 0.01, Spells = "Immolate", ModType = "SpellDamage", },
						[2] = { Effect = 0.02, Spells = "Immolate", ModType = "dotSpellDmgM" },
						[2] = { Effect = 0.02, Spells = "Conflagrate", ModType = "Amount", Value = "Fire and Brimstone" }, },				
	}
end